import { describe, it, expect } from "vitest";
import { mount } from "@vue/test-utils";
import CTBaseButton from "./BaseButton.vue";
import { stub } from "./stub";
import { ButtonSize, ButtonType, ButtonVariant } from "./constants";

const props = {
  ...stub,
};

const { type } = props;

describe("BaseButton", () => {
  it("renders loader", async () => {
    const wrapper = mount(CTBaseButton, { props, isLoading: false });

    expect(wrapper.findComponent({ ref: "loadingRef" }).exists()).toBe(false);
    await wrapper.setProps({ isLoading: true });
    expect(wrapper.findComponent({ ref: "loadingRef" }).exists()).toBe(true);
  });

  it("renders button", async () => {
    const wrapper = mount(CTBaseButton, { props });

    expect(wrapper.html().includes("button")).toBe(true);
  });

  it("emits click", async () => {
    const wrapper = mount(CTBaseButton, {
      props: { ...props, isDisabled: undefined },
    });

    await wrapper.trigger("click");

    expect(wrapper.emitted("click")?.length).toBe(1);

    await wrapper.trigger("click");

    expect(wrapper.emitted("click")?.length).toBe(2);

    await wrapper.trigger("click");
    await wrapper.trigger("click");

    expect(wrapper.emitted("click")?.length).toBe(4);
  });

  it("renders slot", async () => {
    const label = "Slot content";
    const wrapper = mount(CTBaseButton, {
      slots: { default: label! },
    });

    expect(wrapper.text()).toBe(label);
  });

  it("has size class", async () => {
    const wrapper = mount(CTBaseButton, {
      props: { type },
    });

    expect(wrapper.classes(`size-${ButtonSize.MEDIUM}`)).toBe(true);

    await wrapper.setProps({ size: ButtonSize.LARGE });
    expect(wrapper.classes(`size-${ButtonSize.LARGE}`)).toBe(true);
  });

  it("has type class", async () => {
    const wrapper = mount(CTBaseButton);

    expect(wrapper.classes(`type-${ButtonType.PRIMARY}`)).toBe(true);

    await wrapper.setProps({ type: ButtonType.DANGER });
    expect(wrapper.classes(`type-${ButtonType.DANGER}`)).toBe(true);
  });

  it("can be variant", async () => {
    const wrapper = mount(CTBaseButton, {
      props: { ...props, variant: undefined },
    });
    expect(wrapper.classes(`variant-${ButtonVariant.DEFAULT}`)).toBe(true);

    await wrapper.setProps({ variant: ButtonVariant.OUTLINE });
    expect(wrapper.classes(`variant-${ButtonVariant.OUTLINE}`)).toBe(true);
  });

  it("can be disabled", async () => {
    const wrapper = mount(CTBaseButton, {
      props: { ...props, isDisabled: false },
    });
    expect("disabled" in wrapper.attributes()).toBe(false);

    await wrapper.setProps({ isDisabled: true });
    expect("disabled" in wrapper.attributes()).toBe(true);

    await wrapper.trigger("click");

    expect(wrapper.emitted("click")).toBeFalsy();
  });
});
