import { describe, expect, it, vi } from "vitest";
import { stub } from "./stub";
import { mount } from "@vue/test-utils";
import { CTCalendar } from "./index";
import { DateFormat, getDateRange } from "../../utilities";
import {
  add,
  addMonths,
  format,
  isSameDay,
  setDate,
  startOfDay,
} from "date-fns";
import { CTInput } from "../input";
import { CTIcon } from "../icon";
import { CalendarDateRange } from "./constants";
import { i18n } from "../../utilities/i18n";

const props = { ...stub };

describe("Calendar", () => {
  it("can be double", async () => {
    const wrapper = mount(CTCalendar, {
      props: { ...props, isDouble: false },
    });
    expect(wrapper.classes("is-double")).toBe(false);
    expect(wrapper.findAll(".month").length).toBe(1);
    await wrapper.setProps({ isDouble: true });
    expect(wrapper.findAll(".is-double").length).toBe(1);
    expect(wrapper.findAll(".month").length).toBe(2);
  });

  it("render single", async () => {
    const modelValue = new Date();
    const date = 24;
    const wrapper = mount(CTCalendar, {
      props: { ...props, isDouble: false, modelValue },
    });
    expect(wrapper.find(".month .label").text()).toBe(
      i18n.global.d(modelValue, DateFormat.MONTH_YEAR),
    );

    await wrapper.findAll(".month .day")[date - 1].trigger("click");
    expect(
      wrapper.findAll(".month .day")[date - 1].classes("is-range-start-date"),
    ).toBe(true);

    await wrapper.findComponent({ ref: "applyRef" }).trigger("click");

    const expectedDate = startOfDay(setDate(modelValue, date));

    expect(
      isSameDay(
        (wrapper.emitted("update:modelValue") as Date[][])[0][0] as Date,
        expectedDate,
      ),
    ).toBe(true);

    const input = wrapper.findComponent(CTInput);

    const inputDate = addMonths(new Date(), 5);
    await input
      .find({ ref: "inputRef" })
      .setValue(format(inputDate, "dd / MM / yyyy"));

    await input.find({ ref: "inputRef" }).trigger("blur");

    await wrapper.findComponent({ ref: "applyRef" }).trigger("click");

    expect(
      isSameDay(
        (wrapper.emitted("update:modelValue") as Date[][])[1][0] as Date,
        inputDate,
      ),
    ).toBe(true);
  });

  it("render range", async () => {
    const modelValue = [new Date(), new Date()];

    const wrapper = mount(CTCalendar, {
      props: { ...props, isDouble: true, isRange: true, modelValue },
    });

    expect(wrapper.findAll(".month")[0].find(".label").text()).toBe(
      i18n.global.d((modelValue as Date[])[0], DateFormat.MONTH_YEAR),
    );

    expect(wrapper.findAll(".month")[1].find(".label").text()).toBe(
      i18n.global.d(
        addMonths((modelValue as Date[])[1], 1),
        DateFormat.MONTH_YEAR,
      ),
    );

    const rangeStartDate = new Date();
    const rangeEndDate = addMonths(new Date(), 1);

    await wrapper
      .findAll(".month:nth-child(1) .day")
      [rangeStartDate.getDate() - 1].trigger("click");

    expect(
      wrapper
        .findAll(".month:nth-child(1) .day")
        [rangeStartDate.getDate() - 1].classes("is-range-start-date"),
    ).toBe(true);

    await wrapper
      .findAll(".month:nth-child(2) .day")
      [rangeEndDate.getDate() - 1].trigger("click");

    expect(
      wrapper
        .findAll(".month:nth-child(2) .day")
        [rangeEndDate.getDate() - 1].classes("is-range-end-date"),
    ).toBe(true);

    await wrapper.findComponent({ ref: "applyRef" }).trigger("click");

    const [startDate, endDate] = (
      wrapper.emitted("update:modelValue") as Date[][][]
    )[0][0];

    expect(isSameDay(startDate as Date, rangeStartDate)).toBe(true);
    expect(isSameDay(endDate as Date, rangeEndDate)).toBe(true);
  });

  it("future to be disabled", async () => {
    const wrapper = mount(CTCalendar, {
      props: { ...props, isFutureDisabled: true, isDouble: true },
    });

    const today = new Date();

    const tomorrow =
      wrapper.findAll(".month:nth-child(1) .day")[today.getDate()] ||
      wrapper.findAll(".month:nth-child(2) .day")[0];

    expect(tomorrow.classes("is-in-future")).toBe(true);
    await tomorrow.trigger("click");
    expect(wrapper.emitted("update:modelValue")).toBeFalsy();
  });

  it("can be closed by Escape key", async () => {
    const wrapper = mount(CTCalendar, {
      props,
      attachTo: document.body,
    });

    window.dispatchEvent(new KeyboardEvent("keydown", { key: "Escape" }));
    expect(wrapper.emitted().close).toBeTruthy();
  });

  it("can set hover date", async () => {
    const wrapper = mount(CTCalendar, {
      props: { ...props, isRange: true },

      attachTo: document.body,
    });
    const spy = vi.spyOn(wrapper.vm, "onDayClick");

    await wrapper.findAll(".day")[0].trigger("click");
    await wrapper.findAll(".day")[2].trigger("mouseenter");

    expect(wrapper.findAll(".day")[1].classes("is-between-date")).toBe(true);
    expect(wrapper.findAll(".day")[2].classes("is-hover-date")).toBe(true);

    await wrapper.findAll(".day")[2].trigger("click");

    expect(spy).toHaveBeenCalledTimes(2);

    await wrapper.findAll(".day")[2].trigger("click");
    await wrapper.findAll(".day")[0].trigger("mouseenter");

    expect(wrapper.findAll(".day")[1].classes("is-between-date")).toBe(true);
    expect(wrapper.findAll(".day")[0].classes("is-hover-date")).toBe(true);
  });

  it("can can change month", async () => {
    const modelValue = new Date();
    const wrapper = mount(CTCalendar, {
      props: { ...props, isDouble: false, modelValue },

      attachTo: document.body,
    });

    await wrapper
      .find(".month header")
      .findAllComponents(CTIcon)[1]
      .trigger("click");

    expect(wrapper.find(".month header .label").text()).toBe(
      i18n.global.d(addMonths(modelValue, 1), DateFormat.MONTH_YEAR),
    );
  });

  it("sets initial current month correctly", async () => {
    const modelValue = add(new Date(), { days: 5 });
    const wrapper = mount(CTCalendar, {
      props: { ...props, isDouble: false, modelValue },

      attachTo: document.body,
    });

    // @ts-ignore
    expect(wrapper.vm.currentMonth).toEqual(modelValue);

    const arrayModelValue = [new Date(), add(new Date(), { months: 5 })];
    await wrapper.setProps({ modelValue: arrayModelValue });

    // @ts-ignore
    expect(wrapper.vm.currentMonth).toEqual(arrayModelValue[1]);
  });

  it("renders calendar side navigation", async () => {
    const modelValue = add(new Date(), { days: 5 });
    const wrapper = mount(CTCalendar, {
      props: {
        ...props,
        isRange: true,
        hasSidebarNav: undefined,
        modelValue,
      },
    });
    expect(wrapper.find({ ref: "calendarSidebar" }).exists()).toBe(false);

    await wrapper.setProps({ hasSidebarNav: true });
    expect(wrapper.find({ ref: "calendarSidebar" }).exists()).toBe(true);
  });

  it("sets correct sidebar option when clicking sidenav", async () => {
    const wrapper = mount(CTCalendar, {
      props: {
        ...props,
        hasSidebarNav: true,
        isRange: true,
        modelValue: new Date(),
      },
    });

    await wrapper
      .find({ ref: "calendarSidebar" })
      .findAll("button")[0]
      .trigger("click");

    await wrapper.findComponent({ ref: "applyRef" }).trigger("click");

    expect(wrapper.emitted("update:modelValue")?.[0]).toEqual([
      getDateRange(CalendarDateRange.LAST_7),
    ]);

    expect(wrapper.vm.sidenavIndex).toBe(0);

    // Custom option click
    await wrapper
      .find({ ref: "calendarSidebar" })
      .findAll("button")[7]
      .trigger("click");

    expect(wrapper.vm.internalDateList.length).toBe(1);
    expect(wrapper.vm.sidenavIndex).toBe(7);
  });

  it("sets initial sideNavIndex correctly", () => {
    const modelValue = getDateRange(CalendarDateRange.LAST_90);

    const wrapper = mount(CTCalendar, { props: { modelValue } });

    expect(wrapper.vm.sidenavIndex).toBe(2);
  });

  // it("sets sideNavIndex by date range click", async () => {
  //   const modelValue = [new Date(), new Date()];
  //
  //   const wrapper = mount(CTCalendar, {
  //     props: {
  //       modelValue,
  //       isRange: true,
  //       isDouble: false,
  //       hasSidebarNav: true,
  //     },
  //   });
  //
  //   expect(wrapper.vm.sidenavIndex).toBe(5);
  //
  //   const index = new Date().getDate() - 1;
  //
  //   await wrapper.find(".month").findAll(".day")[index].trigger("click");
  //
  //   await wrapper.findComponent({ ref: "previousMonthRef" }).trigger("click");
  //   await wrapper.findComponent({ ref: "previousMonthRef" }).trigger("click");
  //   await wrapper.findComponent({ ref: "previousMonthRef" }).trigger("click");
  //
  //   await wrapper.find(".month").findAll(".day")[index].trigger("click");
  //
  //   expect(wrapper.vm.sidenavIndex).toBe(2);
  // });
});
