import type { Ref } from "vue";
import { computed } from "vue";
import type { ChartData, ChartDataSet, ChartProps } from "../chart";
import { ChartType } from "../chart";

export interface UseChartHelperReturn {
  max: Ref<number>;
  alternativeMax: Ref<number>;
  min: Ref<number>;
  alternativeMin: Ref<number>;
  difference: Ref<number>;
  alternativeDifference: Ref<number>;
  maxDataPoints: Ref<number>;
  hasBarChart: Ref<boolean>;
  hasAlternativeDataSet: Ref<boolean>;
  labelYList: Ref<[string, number][]>;
  alternativeLabelYList: Ref<[string, number][]>;
}

export const useChartHelper = ({
  data,
  labels,
  isLabelShortened,
}: {
  data: Ref<ChartData>;
  labels: Ref<ChartProps["labels"]>;
  isLabelShortened?: Ref<boolean>;
}): UseChartHelperReturn => {
  const maxDataPoints = computed(() =>
    Math.max(
      ...data.value.datasets.map((set: ChartDataSet) => set.data.length),
    ),
  );

  const multiplier = computed(() => {
    return labels.value?.y ? 1.4 : 1;
  });

  const difference = computed(() => max.value - min.value);
  const alternativeDifference = computed(
    () => alternativeMax.value - alternativeMin.value,
  );

  const hasAlternativeDataSet = computed(() =>
    data.value.datasets.some(
      ({ isAlternative }: ChartDataSet) => isAlternative,
    ),
  );
  const hasBarChart = computed(() =>
    data.value.datasets.some(
      ({ type }: ChartDataSet) => type === ChartType.BAR,
    ),
  );
  const hasMultipleBarTypes = computed(() => {
    const map = data.value.datasets
      .map((set: ChartDataSet) => set.type)
      .reduce((types: Map<string, number>, type: ChartType) => {
        if (!types.has(type)) {
          types.set(type, 0);
        }

        types.set(type, types.get(type)! + 1);

        return types;
      }, new Map());

    return map.get(ChartType.BAR)! >= 2;
  });

  const getMax = (datasets: ChartDataSet[]): number => {
    if (data.value.max) {
      return data.value.max;
    }

    if (hasMultipleBarTypes.value) {
      const accumulated = datasets.reduce(
        (map: Map<number, number>, set: ChartDataSet) => {
          set.data.flat(1).forEach((value: number, key: number) => {
            if (!map.has(key)) {
              map.set(key, 0);
            }

            map.set(key, map.get(key)! + value);
          });
          return map;
        },
        new Map(),
      );

      return Math.max(...accumulated.values()) * multiplier.value;
    }

    return (
      Math.max(...datasets.flatMap(({ data }: ChartDataSet) => data).flat(1)) *
      multiplier.value
    );
  };

  const getMin = (datasets: ChartDataSet[]): number => {
    const min = Math.min(
      ...datasets.flatMap(({ data }: ChartDataSet) => data).flat(1),
    );

    if (hasBarChart.value || min >= 0) return 0;

    return min - (max.value - max.value / multiplier.value);
  };

  const alternativeDatasets = computed(() =>
    data.value.datasets.filter((set: ChartDataSet) => set.isAlternative),
  );
  const datasets = computed(() =>
    data.value.datasets.filter((set: ChartDataSet) => !set.isAlternative),
  );

  const max = computed(() => {
    return getMax(datasets.value);
  });

  const alternativeMax = computed(() => {
    return getMax(alternativeDatasets.value);
  });

  const min = computed(() => {
    return getMin(datasets.value);
  });

  const alternativeMin = computed(() => {
    return getMin(alternativeDatasets.value);
  });

  const labelYList = computed(() => {
    return getLabelList(
      difference.value,
      min.value,
      max.value,
      labels.value?.y || 5,
      isLabelShortened?.value,
    );
  });
  const alternativeLabelYList = computed(() => {
    return getLabelList(
      alternativeDifference.value,
      alternativeMin.value,
      max.value,
      labels.value?.y || 5,
      isLabelShortened?.value,
    );
  });

  return {
    min,
    alternativeMin,
    max,
    alternativeMax,
    hasBarChart,
    hasAlternativeDataSet,
    alternativeDifference,
    difference,
    maxDataPoints,
    labelYList,
    alternativeLabelYList,
  };
};

const getLabelList = (
  difference: number,
  min: number,
  max: number,
  length: number,
  isShortened?: boolean,
): [string, number][] => {
  return Array.from({ length }, (x, i) => {
    const value = difference - (difference / length) * i + min;

    if (isShortened && value >= 1_000_000) {
      return [`${(value / 1_000_000).toFixed(1)}m`, value];
    }
    if (isShortened && value >= 1_000) {
      return [`${(value / 1_000).toFixed(1)}k`, value];
    }

    if (max < 50) {
      return [value.toFixed(1), value];
    }

    return [Math.round(value).toString(), value];
  });
};
export const roundRect = (
  ctx: CanvasRenderingContext2D,
  x: number,
  y: number,
  width: number,
  height: number,
  radius: number | { tl: number; tr: number; br: number; bl: number },
  fill: boolean,
  stroke: boolean,
): void => {
  if (typeof stroke === "undefined") {
    stroke = true;
  }

  if (typeof radius === "number") {
    radius = { tl: radius, tr: radius, br: radius, bl: radius };
  } else {
    radius = {
      tl: radius.tl || 0,
      tr: radius.tr || 0,
      br: radius.br || 0,
      bl: radius.bl || 0,
    };
  }

  ctx.beginPath();
  ctx.moveTo(x + radius.tl, y);
  ctx.lineTo(x + width - radius.tr, y);
  ctx.quadraticCurveTo(x + width, y, x + width, y + radius.tr);
  ctx.lineTo(x + width, y + height - radius.br);
  ctx.quadraticCurveTo(
    x + width,
    y + height,
    x + width - radius.br,
    y + height,
  );
  ctx.lineTo(x + radius.bl, y + height);
  ctx.quadraticCurveTo(x, y + height, x, y + height - radius.bl);
  ctx.lineTo(x, y + radius.tl);
  ctx.quadraticCurveTo(x, y, x + radius.tl, y);
  ctx.closePath();
  if (fill) {
    ctx.fill();
  }
  if (stroke) {
    ctx.stroke();
  }
};

export const getEventClient = (
  event: TouchEvent | MouseEvent,
): { clientX: number; clientY: number } => {
  if ((event as TouchEvent).touches) {
    const { clientX, clientY } = (event as TouchEvent).touches[0];
    return {
      clientX,
      clientY,
    };
  }

  const { clientX, clientY } = event as MouseEvent;

  return { clientX, clientY };
};
