import { describe, expect, it } from "vitest";
import { mount } from "@vue/test-utils";
import { CTCloudinaryImage } from "./index";
import { stub } from "./stub";
import { useRootStore } from "../../stores/root";
import { storeToRefs } from "pinia";
import { getCloudinarySrc, getThemedSrc } from "./helpers";
import { ThemeColor } from "../theme";
import { testingPinia } from "../../test";
import { nextTick } from "vue";

const props = { ...stub };
const { alt, src } = props;
const params = { w: 800, h: 400 };
const fileMap = {
  [getThemedSrc(src, ThemeColor.DARK)]: { height: 100, width: 200 },
  [getThemedSrc(src, ThemeColor.LIGHT)]: { height: 100, width: 200 },
  [src]: { height: 100, width: 200 },
};
const folder = "developers-v2";
const cloudName = "chargetrip";

const cloudinary = { fileMap, folder, cloudName };
describe("CloudinaryImage", () => {
  it("renders alt", () => {
    const wrapper = mount(CTCloudinaryImage, {
      props,
      global: { provide: { cloudinary: { ...cloudinary, fileMap: {} } } },
    });

    expect(wrapper.find({ ref: "imgRef" }).attributes("alt")).toBe(alt);
  });

  it("can be svg", () => {
    const wrapper = mount(CTCloudinaryImage, {
      props: { ...props, src: "/api.svg" },
      global: { provide: { cloudinary: { ...cloudinary, fileMap: {} } } },
    });

    expect(wrapper.find({ ref: "svgWrapperRef" }).exists()).toBe(true);
  });

  it("can be themed", async () => {
    const store = useRootStore();
    const { normalizedThemeColor } = storeToRefs(store);

    const expected = getThemedSrc(
      src,
      normalizedThemeColor.value as ThemeColor,
    );

    const wrapper = mount(CTCloudinaryImage, {
      props,
      attachTo: document.body,
      global: {
        provide: {
          cloudinary: {
            ...cloudinary,
            fileMap,
          },
        },
      },
    });

    expect(wrapper.vm.normalizedSrc).toBe(
      getCloudinarySrc({}, cloudName, folder, expected),
    );
  });

  it("can be not themed", async () => {
    const wrapper = mount(CTCloudinaryImage, {
      props,
      attachTo: document.body,
      global: {
        provide: {
          cloudinary: {
            ...cloudinary,
            fileMap: { [src]: { height: 100, width: 200 } },
          },
        },
      },
    });

    expect(wrapper.vm.normalizedSrc).toBe(
      getCloudinarySrc({}, cloudName, folder, src),
    );
  });

  it("can have params", async () => {
    const store = useRootStore(testingPinia);
    const { normalizedThemeColor } = storeToRefs(store);

    const wrapper = mount(CTCloudinaryImage, {
      props,
      global: { provide: { cloudinary } },
    });

    await wrapper.setProps({ params });

    const themedSrc = getThemedSrc(
      src,
      normalizedThemeColor.value as ThemeColor,
    );

    expect(wrapper.vm.normalizedSrc).toBe(
      getCloudinarySrc(params, cloudName, folder, themedSrc),
    );
  });

  it("emits load", async () => {
    const wrapper = mount(CTCloudinaryImage, {
      props,
      global: { provide: { cloudinary } },
    });

    wrapper.find({ ref: "imgRef" }).element.dispatchEvent(new Event("load"));

    await nextTick();

    expect(wrapper.emitted("load")).toBeTruthy();
  });
});
