import { describe, expect, it } from "vitest";
import { mount } from "@vue/test-utils";
import { CTCodeBlock } from "../code-block";
import { CTIcon } from "../icon";
import { CTButton } from "../button";
import { getLines } from "./helpers";
import { cleanHTML } from "../../utilities/vitest";
import { stub } from "./stub";
import { messages } from "./i18n";
import { nextTick } from "vue";
import { useRootStore } from "../../stores/root";
import { ThemeColor } from "../theme";
import { ButtonSize } from "../base-button";

const props = {
  ...stub,
};

const { label, externalUrl, code, language } = props;

describe("CodeBlock", () => {
  it("renders label", async () => {
    const wrapper = mount(CTCodeBlock, { props });

    expect(wrapper.find({ ref: "labelRef" }).text()).toBe(label);
  });

  it("can be inline", async () => {
    const wrapper = mount(CTCodeBlock, { props });

    expect(wrapper.classes("is-inline")).toBeFalsy();

    expect(
      wrapper.findComponent({ ref: "copyRef" }).props("size"),
    ).toStrictEqual(ButtonSize.MEDIUM);

    await wrapper.setProps({ isInline: true });

    expect(wrapper.classes("is-inline")).toBeTruthy();
    expect(
      wrapper.findComponent({ ref: "copyRef" }).props("size"),
    ).toStrictEqual(ButtonSize.SMALL);
  });

  it("renders icon", () => {
    const wrapper = mount(CTCodeBlock, { props });

    expect(wrapper.findComponent(CTIcon).exists()).toBe(true);
  });

  it("renders playground button", async () => {
    const wrapper = mount(CTCodeBlock, {
      props: { ...props, externalUrl: undefined },
    });

    expect(wrapper.findComponent(CTButton).exists()).toBe(false);

    await wrapper.setProps({ externalUrl });

    expect(wrapper.findComponent(CTButton).exists()).toBe(true);
    expect(wrapper.findComponent(CTButton).attributes("href")).toBe(
      externalUrl,
    );
  });

  it("renders code", () => {
    const wrapper = mount(CTCodeBlock, {
      props,
    });

    const expected = cleanHTML(
      `<pre class="flex flex-col pr-6">${getLines(code!, language)
        .map((line) => `<code class="last:pb-4">${line}</code>`)
        .join("")}</pre>`,
    );

    const received = cleanHTML(wrapper.find({ ref: "codeRef" }).html());

    expect(expected).toBe(received);
  });

  it("can be expandable", async () => {
    const wrapper = mount(CTCodeBlock, {
      props: { ...props, isExpandable: true },
    });

    expect(wrapper.find({ ref: "labelRef" }).text()).toBe(messages.en.show);
    // expect(wrapper.find({ ref: "bodyRef" }).isVisible()).toBe(false);
    await wrapper.findComponent({ ref: "expandToggleRef" }).trigger("click");
    expect(wrapper.find({ ref: "labelRef" }).text()).toBe(messages.en.hide);
    // expect(wrapper.find({ ref: "bodyRef" }).isVisible()).toBe(true);
  });

  it("can be inversed", async () => {
    const wrapper = mount(CTCodeBlock, {
      props: { ...props, isInverted: false },
    });

    const store = useRootStore();

    expect(wrapper.classes("is-inverted")).toBe(false);
    await wrapper.setProps({ isInverted: true });
    expect(wrapper.classes("is-inverted")).toBe(true);

    store.setThemeColor(ThemeColor.DARK);

    await wrapper.setProps({ isInverted: { dark: true } });
    expect(wrapper.classes("is-inverted")).toBe(true);

    store.setThemeColor(ThemeColor.LIGHT);
    await nextTick();

    expect(wrapper.classes("is-inverted")).toBe(false);

    await wrapper.setProps({ isInverted: { light: true } });
    expect(wrapper.classes("is-inverted")).toBe(true);
  });

  // it("renders slot", () => {
  //   // TODO: doesn't make any sense... app.config.compilerOptions.whitespace = "preserve" is set, still not picking it up.
  //   const wrapper = mount(CTCodeBlock, {
  //     props: {
  //       ...props,
  //       code: undefined,
  //     },
  //
  //     slots: {
  //       default: code!,
  //     },
  //   });
  //
  //   const expected = cleanHTML(
  //     `<pre class="grid h-full">${getLines(code!, language)
  //       .map((line) => `<code>${line}</code>`)
  //       .join("")}</pre>`
  //   );
  //   const received = cleanHTML(wrapper.find({ ref: "codeRef" }).html());
  //   // console.log("expected", expected);
  //   // console.log("received", received);
  //
  //   // expect(expected).toBe(received);
  // });
});
