import { describe, it, expect } from "vitest";
import { CTFileUpload } from "./index";
import { flushPromises, mount } from "@vue/test-utils";
import { stub } from "./stub";
import { CTIcon, IconName } from "../icon";
import type { BaseValidation } from "@vuelidate/core";

const props = {
  ...stub,
};

const { accept, isMultiple, modelValue } = props;

describe("FileUpload", () => {
  it("renders label", async () => {
    const wrapper = mount(CTFileUpload, {
      props,
    });

    expect(wrapper.find({ ref: "labelRef" }).exists()).toStrictEqual(true);
  });

  it("can be disabled", async () => {
    const wrapper = mount(CTFileUpload, {
      props: {
        ...props,
        modelValue: [],
        isMultiple: false,
      },
    });

    expect(wrapper.classes("is-disabled")).toBeFalsy();

    await wrapper.setProps({ modelValue });

    expect(wrapper.classes("is-disabled")).toBeTruthy();

    wrapper
      .find({ ref: "elRef" })
      .element.dispatchEvent(new Event("dragenter"));

    await flushPromises();
    await wrapper.find({ ref: "elRef" }).trigger("dragenter");

    expect(
      wrapper.findComponent({ ref: "iconDownRef" }).exists(),
    ).toStrictEqual(true);

    expect(wrapper.findComponent({ ref: "iconUpRef" }).exists()).toStrictEqual(
      false,
    );

    await wrapper.setProps({ isMultiple: true });

    expect(wrapper.findComponent({ ref: "iconUpRef" }).exists()).toStrictEqual(
      true,
    );
    expect(
      wrapper.findComponent({ ref: "iconDownRef" }).exists(),
    ).toStrictEqual(false);
  });

  it("renders error", async () => {
    const wrapper = mount(CTFileUpload, {
      props: {
        ...props,
      },
    });

    const errorMessage = "Error message!";

    expect(
      wrapper.findComponent({ ref: "ctLabelRef" }).props("sublabel"),
    ).toBeFalsy();

    expect(
      wrapper.findComponent({ ref: "checkmarkRef" }).props("name"),
    ).toStrictEqual(IconName.MD_NAVIGATION_DONE);

    expect(wrapper.classes("is-error")).toBeFalsy();

    await wrapper.setProps({ errorMessage });

    expect(
      wrapper.findComponent({ ref: "ctLabelRef" }).props("sublabel"),
    ).toStrictEqual(errorMessage);

    expect(
      wrapper.findComponent({ ref: "checkmarkRef" }).props("name"),
    ).toStrictEqual(IconName.MD_NAVIGATION_CLOSE);

    expect(wrapper.classes("is-error")).toBeTruthy();

    const validationErrorMessage = "Validation error message";

    await wrapper.setProps({
      validation: {
        $errors: [{ $message: validationErrorMessage }],
        $error: true,
      } as BaseValidation,
      errorMessage: undefined,
    });

    expect(
      wrapper.findComponent({ ref: "ctLabelRef" }).props("sublabel"),
    ).toStrictEqual(validationErrorMessage);
  });

  //
  // it("can be focused", async () => {
  //   const wrapper = mount(CTFileUpload, { props });
  //   expect(wrapper.classes("is-focused")).toBe(false);
  //   await wrapper.trigger("dragenter");
  //   expect(wrapper.classes("is-focused")).toBe(true);
  //   expect(wrapper.find({ ref: "labelRef" }).text()).toBe(messages.en.release);
  //   await wrapper.trigger("dragleave");
  //   expect(wrapper.classes("is-focused")).toBe(false);
  // });

  it("can limit accept", async () => {
    const wrapper = mount(CTFileUpload, {
      props: { ...props, accept: undefined },
    });

    expect(wrapper.find({ ref: "inputRef" }).attributes("accept")).toBeFalsy();
    await wrapper.setProps({ accept });
    expect(wrapper.find({ ref: "inputRef" }).attributes("accept")).toBe(accept);
  });

  it("can be multiple", async () => {
    const wrapper = mount(CTFileUpload, {
      props: { ...props, isMultiple: undefined },
    });

    expect(
      wrapper.find({ ref: "inputRef" }).attributes("multiple"),
    ).toBeFalsy();

    await wrapper.setProps({ isMultiple });

    expect("multiple" in wrapper.find({ ref: "inputRef" }).attributes()).toBe(
      isMultiple,
    );
  });

  it("renders file list", async () => {
    const wrapper = mount(CTFileUpload, {
      props: { ...props },
    });

    expect(wrapper.find({ ref: "fileListRef" }).findAll("li").length).toBe(
      modelValue.length,
    );
  });

  it("can delete file from list", async () => {
    const wrapper = mount(CTFileUpload, {
      props: { ...props },
    });

    await wrapper
      .find({ ref: "fileListRef" })
      .find("li")
      .findAllComponents(CTIcon)[1]
      .trigger("click");

    expect(wrapper.emitted("update:modelValue")?.[0]?.[0]).toStrictEqual([]);
  });

  it("renders right icon", async () => {
    const wrapper = mount(CTFileUpload, {
      props: { ...props, isMultiple: true, accept: undefined },
    });

    expect(
      wrapper.findComponent({ ref: "iconDownRef" }).exists(),
    ).toStrictEqual(true);

    expect(wrapper.findComponent({ ref: "iconUpRef" }).exists()).toStrictEqual(
      false,
    );

    // wrapper
    //   .find({ ref: "elRef" })
    //   .element.dispatchEvent(new Event("dragenter"));
    //
    // await flushPromises();
    // await wrapper.find({ ref: "elRef" }).trigger("dragenter");
    //
    // expect(
    //   wrapper.findComponent({ ref: "iconDownRef" }).exists()
    // ).toStrictEqual(false);
    //
    // expect(wrapper.findComponent({ ref: "iconUpRef" }).exists()).toStrictEqual(
    //   true
    // );
  });

  it("can upload file", async () => {
    const event = {
      target: {
        files: [
          {
            name: "image.png",
            size: 50000,
            type: "image/png",
          },
        ],
      },
    };
    const wrapper = mount(CTFileUpload, {
      props: { ...props, modelValue: [], accept: undefined, isMultiple: true },

      attachTo: document.body,
    });

    // @ts-ignore
    wrapper.vm.onInput(event);

    expect(
      (wrapper.emitted("update:modelValue") as (typeof Event)[][])[0][0],
    ).toEqual([...event.target.files]);

    await wrapper.setProps({ isMultiple: false, modelValue });

    wrapper.vm.onInput(event as unknown as Event);

    expect(wrapper.emitted("update:modelValue")?.[1]).toBeFalsy();
  });
});
