import { describe, expect, it, vi } from "vitest";
import { mount } from "@vue/test-utils";
import { CTInput, InputType } from "./index";
import { stub } from "./stub";
import { CTIcon, IconName } from "../icon";
import { CTLabel } from "../label";
import IMask from "imask";
import { config } from "../../config";
import { FormControlSize } from "../form-control/constants";

const props = { ...stub };
const { hotkey, label, placeholder, type, maskedOptions } = props;

describe("Input", () => {
  it("renders icon", async () => {
    const wrapper = mount(CTInput, {
      props: { ...props, icon: undefined, hotkey: undefined },
    });

    expect(wrapper.findComponent(CTIcon).exists()).toBe(false);
    await wrapper.setProps({ icon: IconName.MD_CONTROLS_SEARCH });
    expect(wrapper.findComponent(CTIcon).exists()).toBe(true);
  });

  it("renders unit slot", async () => {
    const unit = "KG";
    const wrapper = mount(CTInput, {
      props: { ...props },
      slots: {
        unit,
      },
    });

    expect(wrapper.find({ ref: "unitRef" }).exists()).toBe(true);
    expect(wrapper.find({ ref: "unitRef" }).text()).toBe(unit);
  });

  it("renders hotkey", async () => {
    const wrapper = mount(CTInput, {
      props: { ...props, icon: undefined, hotkey: undefined },
      attachTo: document.body,
    });

    expect(wrapper.findComponent({ ref: "hotkeyRef" }).exists()).toBe(false);
    await wrapper.setProps({ hotkey });
    expect(wrapper.findComponent({ ref: "hotkeyRef" }).exists()).toBe(true);

    // const spy = vi.spyOn(hotkey!, "callback");

    // const event = new KeyboardEvent("keydown", {
    //   key: hotkey!.key,
    // });
    //
    // window.dispatchEvent(event);
    //
    // await nextTick();
    //
    // console.log(expect(spy).toHaveBeenCalledWith("hey"));
  });

  it("renders focus key state", async () => {
    const wrapper = mount(CTInput, {
      props: {
        ...props,
        hotkey: undefined,
        modelValue: "",
        type: InputType.TEXT,
        hasFocusKey: false,
        maskedOptions: undefined,
      },
      attachTo: document.body,
    });

    expect(wrapper.findComponent({ ref: "hotkeyRef" }).exists()).toBe(false);

    await wrapper.setProps({ hasFocusKey: true });
    expect(
      wrapper.findComponent({ ref: "hotkeyRef" }).props("icon"),
    ).toStrictEqual(IconName.SM_KEYBOARD_SLASH);
  });

  it("renders clear hotkey", async () => {
    const wrapper = mount(CTInput, {
      props: {
        ...props,
        hotkey: undefined,
        modelValue: "",
        type: InputType.TEXT,
        hasClear: false,
        maskedOptions: undefined,
      },
      attachTo: document.body,
    });

    await wrapper.setProps({ modelValue: "123" });

    expect(wrapper.findComponent({ ref: "clearRef" }).exists()).toStrictEqual(
      false,
    );

    await wrapper.setProps({ hasClear: true });

    expect(wrapper.findComponent({ ref: "clearRef" }).exists()).toStrictEqual(
      true,
    );

    await wrapper.findComponent({ ref: "clearRef" }).trigger("click");

    expect(wrapper.emitted("update:modelValue")?.[0]?.[0]).toStrictEqual("");
  });

  it("can be debounced", async () => {
    vi.useFakeTimers();
    const wrapper = mount(CTInput, {
      props: {
        ...props,
        hotkey: undefined,
        modelValue: "",
        type: InputType.TEXT,
        isDebounced: false,
        maskedOptions: undefined,
      },
      attachTo: document.body,
    });

    const value = "13";

    await wrapper.find({ ref: "inputRef" }).setValue(value);

    expect(wrapper.emitted("update:modelValue")?.[0]?.[0]).toStrictEqual(value);

    await wrapper.setProps({ isDebounced: true });

    const debouncedValue = "14";
    await wrapper.find({ ref: "inputRef" }).setValue(debouncedValue);

    vi.advanceTimersByTime(config.DEBOUNCE_TIME);

    expect(wrapper.emitted("update:modelValue")?.[1]?.[0]).toStrictEqual(
      debouncedValue,
    );
  });

  it("renders label/placeholder", async () => {
    const wrapper = mount(CTInput, {
      props: {
        ...props,
        size: FormControlSize.LARGE,
        label,
        placeholder: undefined,
        hotkey: undefined,
      },
    });

    expect(wrapper.findComponent(CTLabel).exists()).toBe(true);
    expect(wrapper.findComponent(CTLabel).text()).toBe(label);

    await wrapper.setProps({ size: FormControlSize.SMALL });

    expect(wrapper.findComponent(CTLabel).exists()).toBe(false);
    expect(wrapper.find({ ref: "inputRef" }).attributes("placeholder")).toBe(
      label,
    );

    await wrapper.setProps({ placeholder });

    expect(wrapper.find({ ref: "inputRef" }).attributes("placeholder")).toBe(
      placeholder,
    );
  });

  it("renders label and placeholder", async () => {
    const wrapper = mount(CTInput, {
      props: {
        ...props,
        size: FormControlSize.LARGE,
        label,
        hotkey: undefined,
      },
      slots: {
        inputPrefix: `Works`,
      },
    });

    expect(wrapper.findComponent(CTLabel).exists()).toBe(true);
    expect(wrapper.findComponent(CTLabel).text()).toBe(label);
    expect(wrapper.find({ ref: "inputRef" }).attributes("placeholder")).toBe(
      placeholder,
    );
  });

  it("renders input type", async () => {
    const wrapper = mount(CTInput, {
      props: { ...props, hotkey: undefined },
    });

    expect(wrapper.find({ ref: "inputRef" }).attributes("type")).toBe(type);

    // Necessary for emitting number correctly.
    await wrapper.setProps({ type: InputType.NUMBER });
    expect(wrapper.find({ ref: "inputRef" }).attributes("type")).toBe(
      InputType.TEXT,
    );
  });

  it("renders visible toggle", async () => {
    const wrapper = mount(CTInput, {
      props: { ...props, type: InputType.PASSWORD, hotkey: undefined },
    });

    expect(wrapper.findComponent({ ref: "visibleToggleRef" }).exists()).toBe(
      true,
    );

    expect(wrapper.find({ ref: "inputRef" }).attributes("type")).toBe(
      InputType.PASSWORD,
    );

    await wrapper.findComponent({ ref: "visibleToggleRef" }).trigger("click");

    expect(wrapper.find({ ref: "inputRef" }).attributes("type")).toBe(
      InputType.TEXT,
    );
  });

  it("emits model value", async () => {
    const value = "Value";
    const numberValue = "123";
    const wrapper = mount(CTInput, {
      props: { ...props, maskedOptions: undefined, hotkey: undefined },
    });

    await wrapper.find({ ref: "inputRef" }).setValue(value);

    expect(wrapper.emitted("update:modelValue")?.[0]).toEqual([value]);

    await wrapper.setProps({ type: InputType.NUMBER });
    await wrapper.find({ ref: "inputRef" }).setValue(numberValue);

    expect(wrapper.emitted("update:modelValue")?.[1]).toEqual([
      Number(numberValue),
    ]);

    const newModelValue = 345;
    await wrapper.setProps({ modelValue: newModelValue });

    // expect(
    //   (wrapper.find({ ref: "inputRef" }).element as HTMLInputElement).value
    // ).toBe(newModelValue.toString());

    expect(wrapper.vm.normalizedValue).toBe(newModelValue.toString());
  });

  // TODO: check if we can unit test this https://vueuse.org/core/useFocusWithin
  it("can be focused", async () => {
    const wrapper = mount(CTInput, {
      props: { ...props, isDisabled: false, hotkey: undefined },
    });

    expect(wrapper.classes("is-focused")).toBe(false);
    await wrapper.find({ ref: "inputRef" }).trigger("focus");
    // await nextTick();
    // expect(wrapper.classes("is-focused")).toBe(true);
  });

  it("can be masked", async () => {
    const value = "31012022";
    const wrapper = mount(CTInput, {
      props: {
        ...props,
        hotkey: undefined,
      },
    });

    await wrapper.find({ ref: "inputRef" }).setValue(value);

    expect(wrapper.emitted("update:modelValue")?.[0]).toEqual([
      IMask.createMask(maskedOptions!).resolve(value),
    ]);
  });
});
