import { describe, it, expect } from "vitest";
import { mount } from "@vue/test-utils";
import CTMenu from "./Menu.vue";
import { CTMenuItem } from "../menu";
import { developerPortalStub, dashboardStub } from "./stub";
import { nextTick } from "vue";

const { items } = dashboardStub;

describe("Menu", () => {
  it("can toggle menu", async () => {
    const wrapper = mount(CTMenu, {
      attachTo: document.body,
      props: { ...developerPortalStub },
    });

    expect(wrapper.classes("is-open")).toBe(false);

    await wrapper.findComponent({ ref: "toggleMenuRef" }).trigger("click");

    expect(wrapper.emitted("update:isOpen")?.[0]).toStrictEqual([true]);

    expect(wrapper.classes("is-open")).toBe(true);

    window.dispatchEvent(new Event("click"));

    await nextTick();

    expect(wrapper.classes("is-open")).toBe(false);

    await wrapper.setProps({ isOpen: true });

    expect(wrapper.classes("is-open")).toBe(true);
  });

  it("resizes submenu bg", async () => {
    const wrapper = mount(CTMenu, {
      props: { items, menuIndices: [0, 1] },
      attachTo: document.body,
    });

    global.innerWidth = 1025;
    global.dispatchEvent(new Event("resize"));

    const secondMenuItem = await wrapper
      .find({ ref: "menuNavigationRef" })
      .findAllComponents(CTMenuItem)[1];

    const secondsSubmenu = wrapper.findAll(".submenu")[1].element;

    const width = 500;
    const height = 250;

    Object.defineProperty(secondsSubmenu, "offsetWidth", {
      configurable: true,
      value: width,
    });
    Object.defineProperty(secondsSubmenu, "offsetHeight", {
      configurable: true,
      value: height,
    });

    await secondMenuItem.trigger("mouseenter");

    expect(wrapper.find({ ref: "bgRef" }).html()).include(
      `width: ${width}px; height: ${height}px;`,
    );
  });

  it("renders an empty menu", async () => {
    const wrapper = mount(CTMenu, {
      props: {},
    });

    expect(
      wrapper.find({ ref: "menuNavigationRef" }).findAllComponents(CTMenuItem)
        .length,
    ).toBe(0);

    await wrapper.setProps({ items });

    expect(
      wrapper.find({ ref: "menuNavigationRef" }).findAllComponents(CTMenuItem)
        .length,
    ).toBe(items!.length);
  });

  it("renders sign-in/up", async () => {
    const wrapper = mount(CTMenu);

    expect(wrapper.find({ ref: "signInUpRef" }).exists()).toStrictEqual(true);

    await wrapper.setProps({ isDashboard: true });
    expect(wrapper.find({ ref: "signInUpRef" }).exists()).toStrictEqual(false);
  });

  it("renders submenu", async () => {
    const wrapper = mount(CTMenu, { props: { items } });

    expect(wrapper.find({ ref: "perspectiveItemRef" }).exists()).toStrictEqual(
      false,
    );

    await wrapper.setProps({ menuIndices: [0] });

    expect(wrapper.find({ ref: "perspectiveItemRef" }).exists()).toStrictEqual(
      true,
    );
  });

  it("can open submenu", async () => {
    const wrapper = mount(CTMenu, { props: { items, menuIndices: [0] } });

    expect(
      wrapper
        .find({ ref: "perspectiveItemRef" })
        .find(".submenu")
        .classes("is-active"),
    ).toStrictEqual(false);

    // On mobile/ipad it should open on mouse enter.
    global.innerWidth = 1024;
    global.dispatchEvent(new Event("resize"));

    await wrapper
      .find({ ref: "menuNavigationRef" })
      .findComponent(CTMenuItem)
      .trigger("mouseenter");

    expect(
      wrapper
        .find({ ref: "perspectiveItemRef" })
        .find(".submenu")
        .classes("is-active"),
    ).toStrictEqual(false);

    global.innerWidth = 1025;
    global.dispatchEvent(new Event("resize"));

    await wrapper
      .find({ ref: "menuNavigationRef" })
      .findComponent(CTMenuItem)
      .trigger("mouseenter");

    expect(
      wrapper
        .find({ ref: "perspectiveItemRef" })
        .find(".submenu")
        .classes("is-active"),
    ).toStrictEqual(true);
  });

  it("can be reversed", async () => {
    const wrapper = mount(CTMenu, { props: { items, menuIndices: [0, 1] } });

    expect(wrapper.classes("is-reversed")).toStrictEqual(false);

    await wrapper.trigger("mousemove", {
      clientX: 50,
    });

    await wrapper.trigger("mousemove", {
      clientX: 49,
    });

    expect(wrapper.classes("is-reversed")).toStrictEqual(true);
  });

  it("can close submenu with mouseleave", async () => {
    const wrapper = mount(CTMenu, { props: { items, menuIndices: [0, 1] } });

    await wrapper
      .find({ ref: "menuNavigationRef" })
      .findComponent(CTMenuItem)
      .trigger("mouseenter");

    expect(
      wrapper
        .find({ ref: "perspectiveItemRef" })
        .find(".submenu")
        .classes("is-active"),
    ).toStrictEqual(true);

    await wrapper.find({ ref: "menuNavigationRef" }).trigger("mouseleave");

    expect(
      wrapper
        .find({ ref: "perspectiveItemRef" })
        .find(".submenu")
        .classes("is-active"),
    ).toStrictEqual(false);

    // On mobile/ipad it should do nothing
    await wrapper
      .find({ ref: "menuNavigationRef" })
      .findComponent(CTMenuItem)
      .trigger("mouseenter");

    expect(
      wrapper
        .find({ ref: "perspectiveItemRef" })
        .find(".submenu")
        .classes("is-active"),
    ).toStrictEqual(true);

    global.innerWidth = 1024;
    global.dispatchEvent(new Event("resize"));

    await wrapper.find({ ref: "menuNavigationRef" }).trigger("mouseleave");

    expect(
      wrapper
        .find({ ref: "perspectiveItemRef" })
        .find(".submenu")
        .classes("is-active"),
    ).toStrictEqual(true);

    global.innerWidth = 1025;
    global.dispatchEvent(new Event("resize"));
  });
});
