import {
  CTContentModal,
  CTChapterModal,
  CTStepperModal,
  ModalType,
} from "../modal";
import type {
  ContentModalProps,
  ChapterModalProps,
  StepperModalProps,
} from "../modal";
import type { Meta, StoryObj } from "@storybook/vue3";
import { CTRadio } from "../radio";
import { CTInput } from "../input";
import { chapterModalStub, stepperModalStub } from "./stub";
import { CTListItem } from "../list-item";
import { CTButton } from "../button";
import { ButtonType } from "../base-button";
import { computed, reactive, ref } from "vue";
import useVuelidate from "@vuelidate/core";
import { required, requiredIf } from "@vuelidate/validators";
const mainListItemList = [
  { label: "Introduction", icon: "reports", to: "/1" },
  { label: "Query vehicles", icon: "reports", to: "/2" },
  { label: "Query vehicle details", icon: "reports", to: "/3" },
  { label: "Query premium vehicle details", icon: "reports", to: "/4" },
];
const asideListItemList = [
  { label: "API", icon: "terminal", to: "/5" },
  { label: "Vehicles", icon: "car", to: "/6" },
  { label: "Stations", icon: "charge-stations", to: "/7" },
  { label: "Routes", icon: "route", to: "/8" },
  { label: "Navigation", icon: "enter", to: "/9" },
  { label: "Tile service", icon: "layers", to: "/10" },
  { label: "Tile service", icon: "layers", to: "/10" },
  { label: "Tile service", icon: "layers", to: "/10" },
  { label: "Tile service", icon: "layers", to: "/10" },
  { label: "Tile service", icon: "layers", to: "/10" },
];

const meta: Meta = {
  title: "Modal",
};

export const Chapter: StoryObj<typeof CTChapterModal> = {
  args: {
    ...chapterModalStub,
  },
  render: (args: ChapterModalProps) => ({
    components: { CTChapterModal, CTListItem, CTButton },
    setup() {
      const showChapterModal = ref(false);
      const showAside = ref(false);
      return {
        args,
        asideListItemList,
        mainListItemList,
        showChapterModal,
        showAside,
      };
    },
    template: `<CTButton @click="showChapterModal = !showChapterModal" label="Toggle"/>
    <CTChapterModal :show-aside="showAside" @update:show-aside="showAside = $event" @close="showChapterModal = false" class="test" :is-active="showChapterModal" v-bind="args">
      <template v-slot:asideHeader>
        <h2>
          API Reference
        </h2>
      </template>
      <template v-slot:aside>
        <ul class="flex flex-col gap-y-1">
          <CTListItem v-for="(asideListItem, key) in asideListItemList" :key="key" v-bind="asideListItem"/>
        </ul>
      </template>
      <template v-slot:mainHeader>
        <h2>
          Sections
        </h2>
      </template>
      <template v-slot:main>
        <ul class="flex flex-col gap-y-1">
          <CTListItem v-for="(mainListItem, key) in mainListItemList" :key="key" v-bind="mainListItem"/>
        </ul>
      </template>
    </CTChapterModal>`,
  }),
};

export const Content: StoryObj<typeof CTContentModal> = {
  args: {
    type: ModalType.WARNING,
  },
  render: (args: ContentModalProps) => ({
    components: { CTContentModal, CTButton },
    setup() {
      const showChapterModal = ref(false);
      return { showChapterModal, args };
    },
    template: `<CTButton @click="showChapterModal = !showChapterModal" label="Toggle"/><CTContentModal @close="showChapterModal = false" class="test" :is-active="showChapterModal" v-bind="args">
    <template v-slot:header>
      <h2>
        Header
      </h2>
    </template>
    <p v-for="(i) in 100" :key="i">
      Test
    </p>
    Main content!
    <template v-slot:footer>
      Footer
    </template>
    </CTContentModal>`,
  }),
};

export const Stepper: StoryObj<typeof CTStepperModal> = {
  args: {
    ...stepperModalStub,
  },
  render: (args: StepperModalProps) => ({
    components: { CTStepperModal, CTButton, CTRadio, CTInput },
    setup() {
      const form = reactive({ option: "", step2a: "", step2b: "" });
      const modelValue = ref(0);
      const v = useVuelidate(
        {
          option: { required },
          step2a: {
            requiredIfOption: requiredIf(
              () => modelValue.value === 1 && form.option === "Option 1",
            ),
          },
          step2b: {
            requiredIfOption: requiredIf(
              () => modelValue.value === 1 && form.option === "Option 2",
            ),
          },
        },
        form,
      );
      const label = computed(() => {
        switch (modelValue.value) {
          case 0:
            return "Step 1";
          case 1:
            return form.option === "Option 1" ? "Step 2a" : "Step 2b";
          default:
            return "";
        }
      });
      const showModal = ref(false);

      return {
        args,
        ButtonType,
        label,
        showModal,
        v,
        modelValue,
        form,
      };
    },
    template: `<CTButton @click="showModal = !showModal" label="Toggle"/>
    <CTStepperModal v-if="showModal" @close="showModal = false" v-model="modelValue" :length="2">
      <template #header>
        <h2>
          {{label}}
        </h2>
      </template>
      <template #default="{index}">
          <div class="step-1" v-if="index === 0">
            <CTRadio id="temp" name="Option 1" label="Option 1" v-model="form.option"/>
            <CTRadio id="temp" name="Option 2" label="Option 2" v-model="form.option"/>
          </div>
          <div class="step-2-a h-[1340px]" v-if="index === 1 && form.option === 'Option 1'">
            <CTInput id="temp" v-model="form.step2a" label="X" is-boxed/>
          </div>
          <div class="step-2-b h-[500px]" v-if="index === 1 && form.option === 'Option 2'">
            <CTInput id="temp" v-model="form.step2b" label="X" is-boxed/>
          </div>
      </template>
      <template #footer>
        <CTButton @click="showModal = false" is-rounded :type="ButtonType.PLAIN">
          Link
        </CTButton>
        <div class="flex gap-2">
          <CTButton @click="modelValue--" is-rounded :type="ButtonType.SECONDARY" :is-disabled="!modelValue">
            Previous
          </CTButton>
          <CTButton @click="modelValue++" is-rounded class="ml-auto" :is-disabled="v.$invalid">
            Next
          </CTButton>
        </div>
      </template>
    </CTStepperModal>
  `,
  }),
};

export default meta;
