import { describe, it, expect, afterEach } from "vitest";
import { stub } from "./stub";
import { mount } from "@vue/test-utils";
import { CTSchemaFieldBlock } from "./index";
import slugify from "slugify";
import { router } from "../../utilities/mock-router";

const props = { ...stub };
const { label, sublabel, type, childList } = props;

describe("SchemaFieldBlock", () => {
  afterEach(async () => {
    await router.push("/");
  });

  it("renders id", () => {
    const wrapper = mount(CTSchemaFieldBlock, {
      props: { ...props, idPrefix: "prefix" },
    });

    expect(wrapper.attributes("id")).toBe(`prefix-${slugify(label)}`);
  });

  it("renders title", () => {
    const wrapper = mount(CTSchemaFieldBlock, { props });

    expect(wrapper.find({ ref: "labelRef" }).text()).toBe(label);
  });

  it("renders type", async () => {
    const wrapper = mount(CTSchemaFieldBlock, {
      props: { ...props, type: undefined },
    });

    expect(wrapper.find({ ref: "typeRef" }).exists()).toBe(false);

    await wrapper.setProps({ type });
    expect(wrapper.find({ ref: "typeRef" }).text()).toBe(type);
  });

  it("renders sublabel", () => {
    const wrapper = mount(CTSchemaFieldBlock, { props });

    expect(wrapper.findComponent({ ref: "sublabelRef" }).text()).toBe(sublabel);
  });

  it("renders required tag", async () => {
    const wrapper = mount(CTSchemaFieldBlock, {
      props,
    });

    expect(wrapper.find({ ref: "requiredRef" }).exists()).toBe(false);

    await wrapper.setProps({ isRequired: true, isArgument: false });

    expect(wrapper.find({ ref: "requiredRef" }).exists()).toBe(false);

    await wrapper.setProps({ isArgument: true });

    expect(wrapper.find({ ref: "requiredRef" }).exists()).toBe(true);

    expect(
      wrapper.find({ ref: "requiredRef" }).text().includes("Required"),
    ).toBe(true);
  });

  it("renders deprecated tag", async () => {
    const wrapper = mount(CTSchemaFieldBlock, {
      props,
    });

    expect(wrapper.findComponent({ ref: "deprecatedTagRef" }).exists()).toBe(
      false,
    );

    await wrapper.setProps({ isDeprecated: true });

    expect(wrapper.findComponent({ ref: "deprecatedTagRef" }).exists()).toBe(
      true,
    );
    expect(wrapper.findComponent({ ref: "deprecatedTagRef" }).text()).toBe(
      "Deprecated",
    );
  });

  it("has top-level/child class", async () => {
    const wrapper = mount(CTSchemaFieldBlock, {
      props,
    });

    expect(wrapper.classes("is-top-level")).toBe(true);
    expect(wrapper.classes("is-child")).toBe(false);

    await wrapper.setProps({ isChild: true });

    expect(wrapper.classes("is-top-level")).toBe(false);
    expect(wrapper.classes("is-child")).toBe(true);
  });

  it("can open child list", async () => {
    const wrapper = mount(CTSchemaFieldBlock, {
      props: { ...props, childList: [], argumentList: [] },
    });

    expect(wrapper.classes("has-child-list")).toBe(false);
    expect(wrapper.find({ ref: "childListRef" }).exists()).toBe(false);

    await wrapper.setProps({ childList: [...childList!] });

    expect(wrapper.classes("has-child-list")).toBe(true);
    expect(wrapper.find({ ref: "childListToggleRef" }).exists()).toBe(true);

    await wrapper.find({ ref: "childListToggleRef" }).trigger("click");

    expect(wrapper.find({ ref: "childListRef" }).exists()).toBe(true);
  });

  it("can open argument list", async () => {
    const wrapper = mount(CTSchemaFieldBlock, {
      attachTo: document.body,
      props: { ...props, argumentList: [] },
    });

    expect(wrapper.classes("has-argument-list")).toBe(false);
    expect(wrapper.classes("show-child-list")).toBe(false);
    expect(wrapper.find({ ref: "childListToggleBarRef" }).exists()).toBe(false);

    await wrapper.setProps({ argumentList: [...childList!] });

    expect(wrapper.classes("has-argument-list")).toBe(true);
    expect(wrapper.find({ ref: "childListToggleBarRef" }).exists()).toBe(true);
    expect(wrapper.classes("show-child-list")).toBe(false);

    await wrapper.find({ ref: "childListToggleBarRef" }).trigger("click");

    expect(wrapper.classes("show-child-list")).toBe(true);
    expect(wrapper.find({ ref: "childListRef" }).isVisible()).toBe(true);
  });
});
