import { describe, expect, it } from "vitest";
import { stub } from "./stub";
import { mount } from "@vue/test-utils";
import { CTSearchPalette } from "./index";
import { CTEmptyState } from "../empty-state";
import { CTLabel } from "../label";
import { nextTick } from "vue";
import type { SearchPaletteResult } from "../search-palette";
const props = { ...stub };
const { resultList } = props;

describe("SearchPalette", () => {
  it("renders results", async () => {
    const wrapper = mount(CTSearchPalette, {
      props: { ...props, resultList: [] },
    });

    expect(wrapper.find({ ref: "resultUlRef" }).exists()).toBe(false);

    await wrapper.setProps({ resultList });
    expect(wrapper.find({ ref: "resultUlRef" }).exists()).toBe(true);

    expect(
      wrapper.find({ ref: "resultUlRef" }).findAllComponents(CTLabel).length,
    ).toBe(resultList?.length || 0);
  });

  it("renders empty state", async () => {
    const wrapper = mount(CTSearchPalette, {
      props: { ...props, resultList },
    });

    expect(wrapper.findComponent(CTEmptyState).exists()).toBe(false);
    // https://v1.test-utils.vuejs.org/api/wrapper/trigger.html
    // eslint-disable-next-line

    await wrapper.setProps({ resultList: [] });
    expect(wrapper.findComponent(CTEmptyState).exists()).toBe(true);
  });

  it("can be closed by Escape key", async () => {
    const wrapper = mount(CTSearchPalette, {
      props,

      attachTo: document.body,
    });

    window.dispatchEvent(new KeyboardEvent("keydown", { key: "Escape" }));
    expect(wrapper.emitted().close).toBeTruthy();
  });

  it("can be closed by clicking the faded background", async () => {
    const wrapper = mount(CTSearchPalette, {
      props,

      attachTo: document.body,
    });

    const fade = wrapper.find({ ref: "fadeRef" });
    expect(fade.exists()).toBe(true);

    await fade.trigger("click");

    expect(wrapper.emitted()).toHaveProperty("close");
  });

  it("highlights active item on MouseEnter", async () => {
    const wrapper = mount(CTSearchPalette, {
      props,
      attachTo: document.body,
    });

    const resultList = await wrapper.find({ ref: "resultUlRef" });
    const resultItems = resultList.findAll("li");

    expect(resultItems.length).toBeGreaterThan(1);

    for (const [index, resultItem] of resultItems.entries()) {
      await resultItem.trigger("mouseenter");
      expect(wrapper.vm.hightLightedIndex).toBe(index);
    }
  });

  it("emits by Enter key", async () => {
    const wrapper = mount(CTSearchPalette, {
      props,
      attachTo: document.body,
    });

    window.dispatchEvent(new KeyboardEvent("keydown", { key: "Enter" }));
    expect(
      (wrapper.emitted("select") as SearchPaletteResult[][])[0][0].to,
    ).toBe(resultList![0].to);
  });

  it("emits by click", async () => {
    const index = 1;
    const wrapper = mount(CTSearchPalette, {
      props,
      attachTo: document.body,
    });

    await wrapper.findAll("main li")[index].trigger("click");

    expect(
      (wrapper.emitted("select") as SearchPaletteResult[][])[0][0].to,
    ).toBe(resultList![index].to);
  });

  it("can scroll by arrow keys", async () => {
    const wrapper = mount(CTSearchPalette, {
      props,
      attachTo: document.body,
    });

    // TODO: figure out why this one fails
    // const spy = vi.spyOn(wrapper.vm, "setScrollPosition");
    expect(wrapper.findAll("main li")[0].classes("is-highlighted")).toBe(true);
    window.dispatchEvent(new KeyboardEvent("keydown", { key: "ArrowDown" }));
    await nextTick();

    expect(wrapper.findAll("main li")[1].classes("is-highlighted")).toBe(true);
    window.dispatchEvent(new KeyboardEvent("keydown", { key: "ArrowUp" }));
    await nextTick();
    expect(wrapper.findAll("main li")[0].classes("is-highlighted")).toBe(true);
  });
});
