import { describe, it, expect, vi } from "vitest";
import { mount } from "@vue/test-utils";
import { CTSegment } from "./index";
import { CTSegmentBlock } from "../segment-block";
import { CTIcon, IconName } from "../icon";

const MockResizeObserver = vi.fn().mockImplementation(() => ({
  disconnect: vi.fn(),
  observe: vi.fn(),
}));

// See: https://vitest.dev/guide/mocking.html#globals
vi.stubGlobal("ResizeObserver", MockResizeObserver);

describe("Segment", () => {
  it("renders an empty segment", async () => {
    const wrapper = mount(CTSegment, {
      props: {
        modelValue: 0,
        items: [],
      },
    });

    expect(wrapper.findAllComponents(CTSegmentBlock).length).toBe(0);
  });

  it("renders labels", async () => {
    const items = [
      {
        label: "Label A",
      },
      {
        label: "Label B",
      },
    ];

    const wrapper = mount(CTSegment, {
      props: {
        modelValue: 0,
        items: items,
      },
    });

    const blocks = wrapper.findAllComponents(CTSegmentBlock);
    expect(blocks.length).toBe(items.length);

    blocks.forEach((block, index) => {
      expect(items[index].label).toEqual(block.text());
    });
  });

  it("renders icons", async () => {
    const items = [
      {
        icon: IconName.MD_LAYOUTS_GRAPH_BARS,
      },
      {
        icon: IconName.MD_PRODUCTS_API,
      },
      {
        icon: IconName.MD_SECURITY_IP,
      },
    ];

    const wrapper = mount(CTSegment, {
      props: {
        modelValue: 0,
        items,
      },
    });

    const blocks = wrapper.findAllComponents(CTSegmentBlock);
    expect(blocks.length).toBe(3);

    blocks.forEach((block) => {
      expect(block.findComponent(CTIcon)).toBeTruthy();
    });
  });

  it("emits click event with correct index", async () => {
    const wrapper = mount(CTSegment, {
      props: {
        modelValue: 1,
        items: [
          {
            label: "Label A",
          },
          {
            label: "Label B",
          },
          {
            label: "Label C",
          },
          {
            label: "Label D",
          },
          {
            label: "Label E",
          },
        ],
      },
    });

    const blocks = wrapper.findAllComponents(CTSegmentBlock);

    // Trigger change by clicking arbitrary elements and make sure the
    // emitted indices match.
    const arbitraryIndices = [0, 1, 4, 3, 2];

    for (let i = 0; i < arbitraryIndices.length; i += 1) {
      await blocks[arbitraryIndices[i]].trigger("click");
      expect((wrapper.emitted("update:modelValue") as number[])[i]).toEqual([
        arbitraryIndices[i],
      ]);
    }
  });
});
