import { describe, it, expect, vi } from "vitest";
import { mount } from "@vue/test-utils";
import { CTBaseSelect } from "./index";
import { stub } from "./stub";
import { SelectType } from "./constants";
import { nextTick } from "vue";

const props = { ...stub, modelValue: "" };
const { id, label, optionList } = props;
describe("BaseSelect", () => {
  it("can be type filter", async () => {
    const wrapper = mount(CTBaseSelect, {
      props: { ...props, type: SelectType.FILTER, modelValue: "" },
    });

    expect(
      wrapper.findComponent({ ref: "formControlRef" }).exists(),
    ).toStrictEqual(false);

    expect(wrapper.findComponent({ ref: "chipRef" }).exists()).toStrictEqual(
      true,
    );
    wrapper.findComponent({ ref: "chipRef" }).vm.$emit("remove");

    expect(wrapper.emitted("clear")).toBeTruthy();

    expect(wrapper.findComponent({ ref: "chipRef" }).text()).toStrictEqual(
      label,
    );
  });

  it("can emit input", async () => {
    const wrapper = mount(CTBaseSelect, {
      props: { ...props, type: SelectType.FILTER, modelValue: "" },
    });

    await wrapper.findComponent({ ref: "elRef" }).trigger("click");

    // 0 === normal
    await wrapper
      .findComponent({ ref: "dropdownRef" })
      .findAll(".ct-list-item")[0]
      .trigger("click");

    expect(wrapper.emitted("input")?.[0]).toStrictEqual([0]);

    // 1 === disabled
    await wrapper
      .findComponent({ ref: "dropdownRef" })
      .findAll(".ct-list-item")[1]
      .trigger("click");

    expect(wrapper.emitted("input")?.[1]).toStrictEqual(undefined);
  });

  it("can emit query", async () => {
    const wrapper = mount(CTBaseSelect, {
      props: { ...props, type: SelectType.FILTER, modelValue: "" },
    });

    const query = "query initial";

    expect(wrapper.find({ ref: "queryWrapperRef" }).exists()).toStrictEqual(
      false,
    );

    await wrapper.setProps({ query });

    expect(wrapper.find({ ref: "queryWrapperRef" }).exists()).toStrictEqual(
      true,
    );

    expect(
      wrapper.findComponent({ ref: "queryRef" }).props("modelValue"),
    ).toStrictEqual(query);

    const emitQuery = "query updated";

    await wrapper
      .findComponent({ ref: "queryRef" })
      .vm.$emit("update:modelValue", emitQuery);

    expect(wrapper.emitted("update:query")?.[0]).toStrictEqual([emitQuery]);
  });

  it("passes props to form control", async () => {
    const modelValue = "Model value";
    const wrapper = mount(CTBaseSelect, {
      props: { ...props, modelValue },
    });

    expect(
      wrapper.findComponent({ ref: "formControlRef" }).props(),
    ).toStrictEqual(expect.objectContaining({ id, modelValue }));
  });

  it("renders value slot", async () => {
    const value = "Slot content";

    const wrapper = mount(CTBaseSelect, {
      props,
      slots: {
        value,
        prefix: "",
        suffix: "",
        dropdownStart: "",
        listItemPrefix: "",
        listItemSuffix: "",
      },
    });

    expect(wrapper.find({ ref: "valueRef" }).text()).toStrictEqual(value);
  });

  it("toggles on focus/blur", async () => {
    vi.useFakeTimers();
    const wrapper = mount(CTBaseSelect, {
      props,
      attachTo: document.body,
    });

    expect(
      wrapper.findComponent({ ref: "dropdownRef" }).isVisible(),
    ).toStrictEqual(false);

    await wrapper.find({ ref: "inputRef" }).trigger("focus");

    expect(wrapper.emitted("update:isActive")?.[0]).toStrictEqual([true]);

    await wrapper.setProps({ isActive: true });

    expect(
      wrapper.findComponent({ ref: "dropdownRef" }).isVisible(),
    ).toStrictEqual(true);

    await wrapper.find({ ref: "inputRef" }).trigger("blur");

    await vi.advanceTimersByTimeAsync(100);

    expect(wrapper.emitted("update:isActive")?.[1]).toStrictEqual([false]);
  });

  it("opens on enter", async () => {
    const wrapper = mount(CTBaseSelect, {
      props,
      attachTo: document.body,
    });

    window.dispatchEvent(
      new KeyboardEvent("keydown", {
        key: "Enter",
      }),
    );

    expect(wrapper.emitted("input")).toBeFalsy();

    await wrapper.setProps({ isActive: true });

    window.dispatchEvent(
      new KeyboardEvent("keydown", {
        key: "Enter",
      }),
    );

    await nextTick();

    expect(wrapper.emitted("input")?.[0]).toStrictEqual([0]);
  });

  it("can cycle options with arrow keys", async () => {
    const wrapper = mount(CTBaseSelect, {
      attachTo: document.body,
      props: { ...props, isActive: true },
    });

    await wrapper.find({ ref: "inputRef" }).trigger("focus");

    // @ts-ignore:next-line
    expect(wrapper.vm.activeIndex).toBe(0);

    window.dispatchEvent(
      new KeyboardEvent("keydown", {
        key: "ArrowDown",
      }),
    );

    // next 2 options are header/disabled, so we expect 3.
    // @ts-ignore:next-line
    expect(wrapper.vm.activeIndex).toBe(3);

    window.dispatchEvent(
      new KeyboardEvent("keydown", {
        key: "ArrowUp",
      }),
    );

    // @ts-ignore:next-line
    expect(wrapper.vm.activeIndex).toBe(0);

    window.dispatchEvent(
      new KeyboardEvent("keydown", {
        key: "ArrowUp",
      }),
    );
    // @ts-ignore:next-line
    expect(wrapper.vm.activeIndex).toBe(optionList.length - 1);
  });
});
