import { describe, expect, it } from "vitest";
import { stub } from "./stub";
import { DOMWrapper, mount, VueWrapper } from "@vue/test-utils";
import { CTSlider } from "./index";
const props = { ...stub };
const { min, max, step } = props;

type SliderWrapper = VueWrapper<typeof CTSlider>;
const checkAttributes = async (
  inputList: DOMWrapper<HTMLInputElement>[],
  wrapper: SliderWrapper,
): Promise<void> => {
  inputList.forEach((input) => {
    expect(input.attributes("min")).toStrictEqual("0");
    expect(input.attributes("max")).toStrictEqual("100");
    expect(input.attributes("step")).toStrictEqual("1");
  });

  await wrapper.setProps({ step, min, max });

  inputList.forEach((input) => {
    expect(input.attributes("min")).toStrictEqual(min!.toString());
    expect(input.attributes("max")).toStrictEqual(max!.toString());
    expect(input.attributes("step")).toStrictEqual(step!.toString());
  });
};

describe("Slider", () => {
  it("can be single", async () => {
    const wrapper = mount(CTSlider, {
      props: { isRange: false, modelValue: 0 },
    });

    expect(wrapper.find({ ref: "fromRef" }).exists()).toStrictEqual(true);
    expect(wrapper.find({ ref: "toRef" }).exists()).toStrictEqual(false);

    await checkAttributes(
      [wrapper.find({ ref: "fromRef" })],
      wrapper as unknown as SliderWrapper,
    );
  });

  it("can be range", async () => {
    const wrapper = mount(CTSlider, {
      props: { isRange: true, modelValue: [0, 0] },
    });

    expect(wrapper.find({ ref: "fromRef" }).exists()).toStrictEqual(true);
    expect(wrapper.find({ ref: "toRef" }).exists()).toStrictEqual(true);

    await checkAttributes(
      [wrapper.find({ ref: "toRef" }), wrapper.find({ ref: "fromRef" })],
      wrapper as unknown as SliderWrapper,
    );
  });

  it("emits modelValue", async () => {
    const wrapper = mount(CTSlider, {
      props: { isRange: false, modelValue: 0 },
    });

    const value = 75;

    await wrapper.find({ ref: "fromRef" }).setValue(value);

    expect(wrapper.emitted("update:modelValue")?.[0]?.[0]).toStrictEqual(value);

    await wrapper.setProps({ isRange: true });

    const rangeValue = [25, 75];

    await wrapper.find({ ref: "fromRef" }).setValue(rangeValue[0]);
    await wrapper.find({ ref: "toRef" }).setValue(rangeValue[1]);

    expect(wrapper.emitted("update:modelValue")?.[2]?.[0]).toStrictEqual(
      rangeValue,
    );
  });

  it("can be disabled", async () => {
    const wrapper = mount(CTSlider, {
      props: { isRange: false, modelValue: 0 },
    });

    expect(wrapper.find({ ref: "fromRef" }).attributes("disabled")).toBeFalsy();

    expect(wrapper.classes("is-disabled")).toBeFalsy();

    await wrapper.setProps({ isDisabled: true });

    expect(
      "disabled" in wrapper.find({ ref: "fromRef" }).attributes(),
    ).toBeTruthy();

    expect(wrapper.classes("is-disabled")).toBeTruthy();
  });
});
