import { fork } from "child_process";
import { fileURLToPath } from "url";
import { defineNuxtModule, addPlugin, createResolver } from "@nuxt/kit";

const requiredOptions = [
  "CLOUDINARY_API_KEY",
  "CLOUDINARY_API_SECRET",
  "ALLOWED_FORMATS",
  "CLOUDINARY_CLOUD_NAME",
  "CLOUDINARY_FOLDER",
];
export interface ModuleOptions {
  [key: string]: string;
  CLOUDINARY_API_KEY: string;
  CLOUDINARY_API_SECRET: string;
  ALLOWED_FORMATS: string;
  CLOUDINARY_CLOUD_NAME: string;
  CLOUDINARY_FOLDER: string;
}

export default defineNuxtModule<ModuleOptions>({
  setup(options: ModuleOptions, nuxt) {
    requiredOptions.forEach((key) => {
      if (!options[key]) {
        throw new Error(`Missing option ${key}`);
      }
    });

    const runtimeDir = fileURLToPath(new URL("./runtime", import.meta.url));
    nuxt.options.vue.compilerOptions.whitespace = "preserve";
    nuxt.options.vite.plugins = [...(nuxt.options.vite.plugins || [])];

    nuxt.options.app.head.link = [...(nuxt.options.app.head.link || [])];

    nuxt.options.css.push(
      "@chargetrip/vue3-components/src/assets/styles/defaults.scss",
    );

    const { resolve } = createResolver(import.meta.url);

    addPlugin(resolve(runtimeDir, "cloudinary.plugin"));
    addPlugin(resolve(runtimeDir, "intercom.plugin.client"));

    let child: ReturnType<typeof fork> | null = null;

    const start = (onExit?: () => void): void => {
      child = fork(resolve(runtimeDir, "dev.mjs"));
      if (child.stdin) {
        process.stdin.pipe(child.stdin);
      }
      if (child.stdout) {
        child.stdout.pipe(process.stdout);
      }

      child.on("exit", () => onExit?.());
    };

    nuxt.hook("ready", () => {
      start();
    });

    nuxt.hook("close", () => {
      child?.kill();
    });
  },
});
